-----------------------------------------------------------------------------------------------
-- Client Lua Script for WhisperCatch
-- Copyright (c) NCsoft. All rights reserved
-----------------------------------------------------------------------------------------------
--[[
	Created by Durveg
	
	On new mesage recieved, check if the player is on the list, if it is change color or update number 
	else create new name and sort 
]]
 
require "Window"
require "FriendshipLib"

-----------------------------------------------------------------------------------------------
-- WhisperCatch Module Definition
-----------------------------------------------------------------------------------------------
local WhisperCatch = {} 
 
-----------------------------------------------------------------------------------------------
-- Constants
-----------------------------------------------------------------------------------------------
-- e.g. local kiExampleVariableMax = 999

-----------------------------------------------------------------------------------------------
-- Initialization
-----------------------------------------------------------------------------------------------
function WhisperCatch:new(o)
    o = o or {}
    setmetatable(o, self)
    self.__index = self 

    -- initialize variables here
    return o
end

function WhisperCatch:Init()
	local bHasConfigureFunction = false
	local strConfigureButtonText = ""
	local tDependencies = {
		-- "UnitOrPackageName",
	}
    Apollo.RegisterAddon(self, bHasConfigureFunction, strConfigureButtonText, tDependencies)
end
 
-----------------------------------------------------------------------------------------------
-- WhisperCatch OnLoad
-----------------------------------------------------------------------------------------------
function WhisperCatch:OnLoad()
    -- load our form file
	self.xmlDoc = XmlDoc.CreateFromFile("WhisperCatch.xml")
	
	--------------------------------------------------------------
	--Setting Variables
	--------------------------------------------------------------
	self.DisplayAWSetting = true
	self.bShowTimestamp = true
	self.strFontSize = ""
	
	self.TimesLoaded = 1
	
	--self.xmlDoc:RegisterCallback("OnDocLoaded", self)
	Apollo.RegisterTimerHandler("Wait", "WaitForIt", self)
	Apollo.CreateTimer("Wait", 10, false)
end

function WhisperCatch:WaitForIt()
	self.xmlDoc:RegisterCallback("OnDocLoaded", self)
end

-----------------------------------------------------------------------------------------------
-- WhisperCatch OnDocLoaded
-----------------------------------------------------------------------------------------------
function WhisperCatch:OnDocLoaded()
	if self.xmlDoc ~= nil and self.xmlDoc:IsLoaded() then
		
		--------------------------------------------------------------
		--ChatLog Addon
		--------------------------------------------------------------

		self.ChatAddon = Apollo.GetAddon("ChatLog")

		if self.ChatAddon == nil then
			self.ChatAddon = Apollo.GetAddon("BetterChatLog")
		end
		
		if self.ChatAddon == nil then
			self.ChatAddon = Apollo.GetAddon("ChatLogEX")
		end
		
		if self.ChatAddon == nil then
			self.ChatAddon = Apollo.GetAddon("ChatFixed")
		end
		
		if self.ChatAddon == nil then
			self.ChatAddon = Apollo.GetAddon("ImprovedChatLoged")
		end

		if self.ChatAddon == nil and self.TimesLoaded <= 5 then
			self.TimesLoaded = self.TimesLoaded + 1
			Apollo.CreateTimer("Wait", 10, false)
			return
		elseif self.ChatAddon == nil and self.TimesLoaded > 5 then
			Apollo.AddAddonErrorText(self, "This addon currently only supports the default ChatLog, BetterChatLog, ChatLogEx, ImprovedChatLog, and ChatFixed. If you have a different chatlog addon installed please let the developer of WhisperCatch know.")
			Apollo.AddAddonErrorText(self, "\\/ Ignore this Error \\/")
			self.wndMain:Show(true)
			return
		end
		
		--------------------------------------------------------------
		--Window Forms
		--------------------------------------------------------------
	    self.wndMain = Apollo.LoadForm(self.xmlDoc, "WhisperCatchForm", nil, self)
		self.wndCont = self.wndMain:FindChild("NameContainer")
		self.wndSetting = self.wndMain:FindChild("Settings")
		
		if self.wndMain == nil then
			Apollo.AddAddonErrorText(self, "Could not load the main window for some reason.")
			return
		end
		
		Apollo.RegisterTimerHandler("CheckList", "CheckList", self)
		
		--------------------------------------------------------------
		--Event Listener
		--------------------------------------------------------------
		Apollo.RegisterEventHandler("ChatMessage", "OnChatMessage", self)
		
		--------------------------------------------------------------
		--Form arrays
		--------------------------------------------------------------
		self.tTextBoxes = {}
		self.tNames = {}
		self.tChatLines = {}
		self.tChatBoxes = {}
		
		local MainChat = self:GetInputWnd()
	
		self.btnShow = Apollo.LoadForm(self.xmlDoc, "ShowList", MainChat, self)
		self.btnShow:Show(true)
		self.btnShow:FindChild("ShowWindow"):SetCheck(true)
		--------------------------------------------------------------
		--Show Forms
		--------------------------------------------------------------
		if self.locSavedWindowLoc then
			self.wndMain:MoveToLocation(self.locSavedWindowLoc)
		end

		self.wndSetting:Show(false)
		self.wndMain:FindChild("SettingsButton"):Show(false)
		Apollo.CreateTimer("CheckList", .25, true) 
	end
end

function WhisperCatch:CheckList()
	local NumNames = 0
	for idx, value in pairs(self.tNames) do
		NumNames = NumNames + 1
	end
	
	if NumNames > 0 then
		self.wndMain:Show(true)
		self.btnShow:FindChild("ShowWindow"):SetCheck(true)
	else
		self.wndMain:Show(false)
		self.btnShow:FindChild("ShowWindow"):SetCheck(false)
	end
end

function WhisperCatch:OnSave(eType)
	if eType ~= GameLib.CodeEnumAddonSaveLevel.Account then
		return
	end
	
	local locWindowLocation = self.wndMain and self.wndMain:GetLocation() or self.locSavedWindowLoc
	
	local tSaved = 
	{
		tWindowLocation = locWindowLocation and locWindowLocation:ToTable() or nil,
		nSaveVersion = knSaveVersion,
		bDisplaySetting = self.DisplayAWSetting,
	}
	
	return tSaved
end

function WhisperCatch:OnRestore(eType, tSavedData)
	self.tSavedData = tSavedData
	if not tSavedData or tSavedData.nSaveVersion ~= knSaveVersion then
		return
	end

	if tSavedData.bDisplaySetting ~= nil then	
		self.DisplayAWSetting = tSavedData.bDisplaySetting
	end
	
	if tSavedData.tWindowLocation then
		self.locSavedWindowLoc = WindowLocation.new(tSavedData.tWindowLocation)
		
		if self.wndMain then
			self.wndMain:MoveToLocation(self.locSavedWindowLoc)
		end
	end
end

-----------------------------------------------------------------------------------------------
-- WhisperCatch Functions
-----------------------------------------------------------------------------------------------
--------------------------------------------------------------
--Called whenever a new message in the chat appears, used to 
--parse out a recieved or sent whisper to send into/ update the list
--------------------------------------------------------------
function WhisperCatch:OnChatMessage(channelCurrent, tMessage)
    local eChannelType = channelCurrent:GetType()

	--If the message is from the player
	if tMessage.bSelf == true then
		Apollo.CreateTimer("CheckList", .25, true)
		--If it is a whisper
		if eChannelType == ChatSystemLib.ChatChannel_Whisper then
			--Static variables 
			local strToFrom = "to"
			local strTypeChat = "Whisper"
			local strRealmName = "" --Normal whisper, no realm needed
			local strAccName = ""		
			local strText = self:GetMessage(tMessage.arMessageSegments)
			
			--If the player the message was sent to is not on the list dont do anything
			if self.tNames[tMessage.strSender] ~= nil then
				self:GenerateMessage(tMessage.strSender, strRealmName, strText, strTypeChat, strToFrom)
			
				self:RecolorName_Sent(tMessage.strSender, strAccName, strRealmName, false)
			else
				self:GenerateMessage(tMessage.strSender, strRealmName, strText, strTypeChat, strToFrom)
				
				self:NewSentName(tMessage.strSender, strAccName, strRealmName, false)
			end
		--If it is an account whisper
		elseif eChannelType == ChatSystemLib.ChatChannel_AccountWhisper then
			local strAccName = ""
			local strRealmName = ""
			local strToFrom = "to"
			local strTypeChat = "AccountWhisper"
			local strText = self:GetMessage(tMessage.arMessageSegments)
			local bMergeAccRealm = false

			--If the player the message was sent to is not on the list dont do anything
			if self.tNames[tMessage.strSender] ~= nil then
				--Get the account name and realm to be used in generate message
				strAccName = self.tNames[tMessage.strSender]:FindChild("AccountName"):GetText()
				strRealmName = self.tNames[tMessage.strSender]:FindChild("AccountName"):GetData()
				
				if (tMessage.strRealmName ~= nil or tMessage.strRealmName ~= "") and (strRealmName == tMessage.strRealmName) then
					bMergeAccRealm = true
				end
						
				self:GenerateMessage(strAccName, strRealmName, strText, strTypeChat, strToFrom)
				
				--Will be adding the setting to switch how names are displayed to name@server
				--the call will go here and be passed to Recolor if the setting is true
				self:RecolorName_Sent(tMessage.strSender, strAccName, strRealmName, bMergeAccRealm)
			else
				local strAccName = nil
				local strRealmName = ""
				local tAccountFriends = FriendshipLib.GetAccountList()
				for idx, tAccountFriend in pairs(tAccountFriends) do
					if tAccountFriend.arCharacters ~= nil then
						for idx, tCharacter in pairs(tAccountFriend.arCharacters) do
							if tCharacter.strCharacterName == tMessage.strSender then
									strAccName = tAccountFriend.strCharacterName
									strRealmName = tCharacter.strRealm
							end
						end
					end
				end
				
				if (tMessage.strRealmName ~= nil or tMessage.strRealmName ~= "") and (strRealmName == tMessage.strRealmName) then
					bMergeAccRealm = true
				end
				
				if strAccName ~= nil and (strRealmName ~= nil or strRealmName ~= "") then
					self:GenerateMessage(strAccName, strRealmName, strText, strTypeChat, strToFrom)
				else 
					self:GenerateMessage(tMessage.strSender, strRealmName, strText, strTypeChat, strToFrom)
				end
				
				self:NewSentName(tMessage.strSender, strAccName, strRealmName, bMergeAccRealm)
			end
		end
		
	--If the message was recieved by the player 		
	else
		Apollo.CreateTimer("CheckList", .25, true)
		--If it is a whisper
    	if eChannelType == ChatSystemLib.ChatChannel_Whisper then
			local strToFrom = "from"
			local strTypeChat = "Whisper"
			local strRealmName = "" 
			local strText = self:GetMessage(tMessage.arMessageSegments)
			
			local tNameBox = self.tNames[tMessage.strSender]
			local strAccName = ""
			--[[
			if tNameBox ~= nil then
				strAccName = tNameBox:FindChild("AccountName"):GetText()
				strRealmName = tNameBox:FindChild("AccountName"):GetData()
			end 
			]]
			self:GenerateMessage(tMessage.strSender, strRealmName, strText, strTypeChat, strToFrom)

			self:CheckNewName(tMessage.strSender, strAccName, strRealmName, false)
		--If it is an account whisper
		elseif eChannelType == ChatSystemLib.ChatChannel_AccountWhisper then
			local strToFrom = "from"
			local strTypeChat = "AccountWhisper"
			local strText = self:GetMessage(tMessage.arMessageSegments)
			local bMergeAccRealm = false
			
			--Sorts through the accountfriend list to get the accname and realm of the player
			local strAccName = nil
			local strRealmName = nil
			local tAccountFriends = FriendshipLib.GetAccountList()
			for idx, tAccountFriend in pairs(tAccountFriends) do
				if tAccountFriend.arCharacters ~= nil then
					for idx, tCharacter in pairs(tAccountFriend.arCharacters) do
						if tCharacter.strCharacterName == tMessage.strSender 
						and (tMessage.strRealmName:len() == 0 
						or tCharacter.strRealm == tMessage.strRealmName) then
							if not tMessage.bSelf 
							or (tPreviousWhisperer 
							and tPreviousWhisperer.strCharacterName == tMessage.strSender) then
								strAccName = tAccountFriend.strCharacterName
								strRealmName = tCharacter.strRealm
							end
						end
					end
				end
			end

			if (tMessage.strRealmName ~= nil or tMessage.strRealmName ~= "") and (strRealmName == tMessage.strRealmName) then
				bMergeAccRealm = true
			end
			
			if strAccName ~= nil and strRealmName ~= nil then
				self:GenerateMessage(strAccName, strRealmName, strText, strTypeChat, strToFrom)
			else 
				self:GenerateMessage(tMessage.strSender, "", strText, strTypeChat, strToFrom)
			end
			
			self:CheckNewName(tMessage.strSender, strAccName, strRealmName, bMergeAccRealm)
   		end
	end
end

--------------------------------------------------------------
--Recolor a name green after replying to it
--------------------------------------------------------------
function WhisperCatch:RecolorName_Sent(strName, strAccName, strRealmName, bMergeName)
	if	self.tNames[strName] ~= nil then
		local strMergedName = strAccName
		
		if bMergeName == true then
			strMergedName = strAccName .. "@" .. strRealmName
		end
		--Used to write the number missed next to the name
		self.tNames[strName]:FindChild("ExtraData"):SetData(1) 

		--Display the name alone after replying
		self.tNames[strName]:FindChild("DisplayName"):SetText(strName) 
		self.tNames[strName]:FindChild("AccountDisplayName"):SetText(strMergedName)
		--Set text color to green
		self.tNames[strName]:FindChild("DisplayName"):SetTextColor("Green")
		self.tNames[strName]:FindChild("AccountDisplayName"):SetTextColor("Green")
		self.tNames[strName]:FindChild("ExtraData"):SetText("Green")
		
		--Rearrange list
		self:RearrangeRecolor()
		self.wndCont:ArrangeChildrenVert()
		self.wndCont:SetVScrollPos(0)
	end
end

--------------------------------------------------------------
--Check if the name is already on the list
--------------------------------------------------------------
function WhisperCatch:CheckNewName(strName, strAccName, strRealmName, bMergeName)
	--If name is not on the list create a new name
	if self.tNames[strName] == nil then
		self:NewPlayerMsg(strName, strAccName, strRealmName, bMergeName)
	--If name is already on the list update it
	else
		self:UpdateName(strName, strAccName, strRealmName, bMergeName)	
	end
end

--------------------------------------------------------------
--Creates a new set of data to be held in the array after 
--recieving a new message 
--------------------------------------------------------------
function WhisperCatch:NewPlayerMsg(strName, strAccName, strRealmName, bMergeName)
	self.tNames[strName] = Apollo.LoadForm(self.xmlDoc, "NameObject", self.wndMain:FindChild("NameContainer"), nil)
	local strColor = "Red"
	local strMergedName = strAccName
	
	if strAccName ~= "" then
		strColor = "xkcdVibrantPurple"
	end
	
	if bMergeName == true then
		strMergedName = strAccName .. "@" .. strRealmName
	end
	
	--Set text to name in red and data to name because text will change
	--self.tNames[strName]:FindChild("DisplayName"):SetText(strName)
	self.tNames[strName]:FindChild("DisplayName"):SetTextColor(strColor)
	self.tNames[strName]:FindChild("DisplayName"):SetData(strName)
	--Store color and number of missed msgs in extra data
	self.tNames[strName]:FindChild("ExtraData"):SetText(strColor)
	self.tNames[strName]:FindChild("ExtraData"):SetData(2)
	--Store Account name and realm in account name
	self.tNames[strName]:FindChild("AccountName"):SetText(strAccName)
	self.tNames[strName]:FindChild("AccountDisplayName"):SetText(strMergedName)
	self.tNames[strName]:FindChild("AccountDisplayName"):SetTextColor(strColor)
	self.tNames[strName]:FindChild("AccountName"):SetData(strRealmName)
	
	--If the number missed is more than 1 start putting it next to the name 
	self.tNames[strName]:FindChild("DisplayName"):SetText(strName .. " (" .. "1" .. ")" )
	if strMergedName ~= "" then
		self.tNames[strName]:FindChild("AccountDisplayName"):SetText(strMergedName .. " (" .. "1" .. ")" )
	else
		self.tNames[strName]:FindChild("AccountDisplayName"):SetText("")
	end

	
	--Rearrange setting this name to top of the list
	self:Rearrange(strName)
	self.wndCont:ArrangeChildrenVert()
	self.wndCont:SetVScrollPos(0)
end

--------------------------------------------------------------
--Updates the data held in the array after recieving a new 
--message from some one on the list
--------------------------------------------------------------
function WhisperCatch:UpdateName(strName, strAccName, strRealmName, bMergeName)
	local nOldNumMissed = self.tNames[strName]:FindChild("ExtraData"):GetData()
	local strColor = "Red"
	local strMergedName = strAccName
	
	if strAccName ~= "" then
		strColor = "xkcdVibrantPurple"
	end
	
	if bMergeName == true then
		strMergedName = strAccName .. "@" .. strRealmName
	end
	
	--If the number missed is more than 1 start putting it next to the name 
	if nOldNumMissed >= 1 then 
		self.tNames[strName]:FindChild("DisplayName"):SetText(strName .. " (" .. nOldNumMissed .. ")")
		if strMergedName ~= "" then
			self.tNames[strName]:FindChild("AccountDisplayName"):SetText(strMergedName .. " (" .. nOldNumMissed .. ")")
		else
			self.tNames[strName]:FindChild("AccountDisplayName"):SetText("")
		end
	else
		self.tNames[strName]:FindChild("DisplayName"):SetText(strName)
		self.tNames[strName]:FindChild("AccountDisplayName"):SetText(strMergedName)
	end
	
	--Update color if it was changed to green, update data not needed but will keep for now
	self.tNames[strName]:FindChild("DisplayName"):SetTextColor(strColor)
	self.tNames[strName]:FindChild("DisplayName"):SetData(strName)
	--Update color and increase the number missed
	self.tNames[strName]:FindChild("ExtraData"):SetText(strColor)
	self.tNames[strName]:FindChild("ExtraData"):SetData(nOldNumMissed + 1)
	--Store Account name and realm
	self.tNames[strName]:FindChild("AccountName"):SetText(strAccName)
	self.tNames[strName]:FindChild("AccountDisplayName"):SetTextColor(strColor)
	self.tNames[strName]:FindChild("AccountName"):SetData(strRealmName)
	
	--Rearrange setting this name to top of the list	
	self:Rearrange(strName)
	self.wndCont:ArrangeChildrenVert()
	self.wndCont:SetVScrollPos(0)
	
	-- If the player had sent an account whisper and then a normal whisper switch the shown name
	if strAccName == "" and strRealmName == "" then
		self.tNames[strName]:FindChild("AccountDisplayName"):Show(false)
		self.tNames[strName]:FindChild("DisplayName"):Show(true)
	end
end

function WhisperCatch:NewSentName(strName, strAccName, strRealmName, bMergeName)
	self.tNames[strName] = Apollo.LoadForm(self.xmlDoc, "NameObject", self.wndMain:FindChild("NameContainer"), nil)
	local strMergedName = strAccName

	if bMergeName == true then
		strMergedName = strAccName .. "@" .. strRealmName
	end
	
	--Set text to name in red and data to name because text will change
	self.tNames[strName]:FindChild("DisplayName"):SetText(strName)
	self.tNames[strName]:FindChild("DisplayName"):SetTextColor("Green")
	self.tNames[strName]:FindChild("DisplayName"):SetData(strName)
	--Store color and number of missed msgs in extra data
	self.tNames[strName]:FindChild("ExtraData"):SetText("Green")
	self.tNames[strName]:FindChild("ExtraData"):SetData(1)
	--Store Account name and realm in account name
	self.tNames[strName]:FindChild("AccountDisplayName"):SetText(strMergedName)
	self.tNames[strName]:FindChild("AccountDisplayName"):SetTextColor("Green")
	self.tNames[strName]:FindChild("AccountName"):SetText(strAccName)
	self.tNames[strName]:FindChild("AccountName"):SetData(strRealmName)
	
	--Rearrange setting this name to top of the list
	self:RearrangeRecolor(strName)
	self.wndCont:ArrangeChildrenVert()
	self.wndCont:SetVScrollPos(0)
end
--------------------------------------------------------------
--General rearrange placing the newest name untop of the list
--------------------------------------------------------------
function WhisperCatch:Rearrange(strNewName)
	local tNewNames = {}
	local nRunning = 1
	
	--For every name in the array
	for idx, value in pairs(self.tNames) do 
		--If the name is the last one that has been sent place it to number 1 in the new array
		if self.tNames[idx]:FindChild("DisplayName"):GetData() == strNewName then
			
			tNewNames[1] = Apollo.LoadForm(self.xmlDoc, "NameObject", self, nil)
			tNewNames[1]:Show(false)
			
			self:CopyData(tNewNames[1], self.tNames[idx])
			self.tNames[idx]:Destroy()--Might not be needed but keeping for now
			self.tNames[idx] = nil
		--If it isnt the current name place it in the 2-n part of the array 
		else
			nRunning = nRunning + 1
			tNewNames[nRunning] = Apollo.LoadForm(self.xmlDoc, "NameObject", self, nil)
			tNewNames[nRunning]:Show(false)
			
			self:CopyData(tNewNames[nRunning], self.tNames[idx])
			self.tNames[idx]:Destroy()
			self.tNames[idx] = nil				
		end
	end
	
	--Reset all data in the array so the number placed into the array will update properly
	self.tNames = {}

	--Place names into array in new order
	for idx = 1, (nRunning), 1 do
		local DisplayName = tNewNames[idx]:FindChild("DisplayName"):GetData()
		self.tNames[DisplayName] = Apollo.LoadForm(self.xmlDoc, "NameObject", self.wndMain:FindChild("NameContainer"), nil)
		
		self:CopyData(self.tNames[DisplayName], tNewNames[idx])
		
		if self.DisplayAWSetting == true and self.tNames[DisplayName]:FindChild("AccountDisplayName"):GetText() ~= "" then
			self.tNames[DisplayName]:FindChild("AccountDisplayName"):Show(true)
			self.tNames[DisplayName]:FindChild("DisplayName"):Show(false)
		else
			self.tNames[DisplayName]:FindChild("AccountDisplayName"):Show(false)
			self.tNames[DisplayName]:FindChild("DisplayName"):Show(true)
		end
		
		tNewNames[idx]:Destroy()
		tNewNames[idx] = nil
	end
	tNewNames = {}
end

--------------------------------------------------------------
--Sorts all red(unreplied) names to the top and green(replied)
--names to the bottom after replying to someone
--------------------------------------------------------------
function WhisperCatch:RearrangeRecolor()
	local tNewRed = {}
	local tNewGrn = {}
	local nRed = 0
	local nGrn = 0
	
	--For all names in the array
	for idx, value in pairs(self.tNames) do
		--If red text place in red array
		if self.tNames[idx]:FindChild("ExtraData"):GetText() == "Red" or 
		   self.tNames[idx]:FindChild("ExtraData"):GetText() == "xkcdVibrantPurple" then
		
			nRed = nRed + 1
			tNewRed[nRed] = Apollo.LoadForm(self.xmlDoc, "NameObject", self, nil)
			tNewRed[nRed]:Show(false)
			
			self:CopyData(tNewRed[nRed], self.tNames[idx])
			
			self.tNames[idx]:Destroy()
			self.tNames[idx] = nil
		--If green text place in green array
		else
			nGrn = nGrn + 1
			tNewGrn[nGrn] = Apollo.LoadForm(self.xmlDoc, "NameObject", self, nil)
			tNewGrn[nGrn]:Show(false)
			
			self:CopyData(tNewGrn[nGrn], self.tNames[idx])
			
			self.tNames[idx]:Destroy()
			self.tNames[idx] = nil
		end
	end

	--Reset data
	self.tNames = {}
	
	--Place all red into array first
	for idx = 1, nRed, 1 do
		local DisplayName = tNewRed[idx]:FindChild("DisplayName"):GetData()
		self.tNames[DisplayName] = Apollo.LoadForm(self.xmlDoc, "NameObject", self.wndMain:FindChild("NameContainer"), nil)
		
		self:CopyData(self.tNames[DisplayName], tNewRed[idx])
		
		if self.DisplayAWSetting == true and self.tNames[DisplayName]:FindChild("AccountDisplayName"):GetText() ~= "" then
			self.tNames[DisplayName]:FindChild("AccountDisplayName"):Show(true)
			self.tNames[DisplayName]:FindChild("DisplayName"):Show(false)
		else
			self.tNames[DisplayName]:FindChild("AccountDisplayName"):Show(false)
			self.tNames[DisplayName]:FindChild("DisplayName"):Show(true)
		end
		
		tNewRed[idx]:Destroy()
		tNewRed[idx] = nil
	end
	tNewRed = {}
	--Place all green into array second
	for idx = 1, nGrn, 1 do
		local DisplayName = tNewGrn[idx]:FindChild("DisplayName"):GetData()
		self.tNames[DisplayName] = Apollo.LoadForm(self.xmlDoc, "NameObject", self.wndMain:FindChild("NameContainer"), nil)
		
		self:CopyData(self.tNames[DisplayName], tNewGrn[idx])
		
		if self.DisplayAWSetting == true and self.tNames[DisplayName]:FindChild("AccountDisplayName"):GetText() ~= "" then
			self.tNames[DisplayName]:FindChild("AccountDisplayName"):Show(true)
			self.tNames[DisplayName]:FindChild("DisplayName"):Show(false)
		else
			self.tNames[DisplayName]:FindChild("AccountDisplayName"):Show(false)
			self.tNames[DisplayName]:FindChild("DisplayName"):Show(true)
		end
		
		tNewGrn[idx]:Destroy()
		tNewGrn[idx] = nil
	end
	tNewGrn = {}
end

--------------------------------------------------------------
--Sorts all names up when removing a name from the list
--------------------------------------------------------------
function WhisperCatch:RearrangeRemove(strRemovedName)
	local tNewNames = {}
	local nRunning = 0
	
	--For all names in the array
	for idx, value in pairs(self.tNames) do 
		--If it is not the name being deleted place in new array
		if self.tNames[idx]:FindChild("DisplayName"):GetData() ~= strRemovedName then

			nRunning = nRunning + 1
			tNewNames[nRunning] = Apollo.LoadForm(self.xmlDoc, "NameObject", self, nil)
			tNewNames[nRunning]:Show(false)
			
			self:CopyData(tNewNames[nRunning], self.tNames[idx])
			self.tNames[idx]:Destroy()
			self.tNames[idx] = nil
		--If it is, delete
		else
			self.tNames[idx]:Destroy()
			self.tNames[idx] = nil
		end
	end
	
	--Reset data
	self.tNames = {}
	
	--Place data back in new order
	for idx = 1, nRunning, 1 do
		local DisplayName = tNewNames[idx]:FindChild("DisplayName"):GetData()
		self.tNames[DisplayName] = Apollo.LoadForm(self.xmlDoc, "NameObject", self.wndCont, nil)
		
		self:CopyData(self.tNames[DisplayName], tNewNames[idx])
		
		if self.DisplayAWSetting == true and self.tNames[DisplayName]:FindChild("AccountDisplayName"):GetText() ~= "" then
			self.tNames[DisplayName]:FindChild("AccountDisplayName"):Show(true)
			self.tNames[DisplayName]:FindChild("DisplayName"):Show(false)
		else
			self.tNames[DisplayName]:FindChild("AccountDisplayName"):Show(false)
			self.tNames[DisplayName]:FindChild("DisplayName"):Show(true)
		end
		
		tNewNames[idx]:Destroy()
		tNewNames[idx] = nil
	end
	tNewNames = {}
end

--------------------------------------------------------------
--Coppies from 1 array of NameObject into another, used for 
--rearranging the names in the window
--------------------------------------------------------------
function WhisperCatch:CopyData(tNew, tOld)
	tNew:FindChild("DisplayName"):SetText(tOld:FindChild("DisplayName"):GetText())
	tNew:FindChild("DisplayName"):SetTextColor(tOld:FindChild("ExtraData"):GetText())
	tNew:FindChild("DisplayName"):SetData(tOld:FindChild("DisplayName"):GetData())
	tNew:FindChild("AccountName"):SetText(tOld:FindChild("AccountName"):GetText())
	tNew:FindChild("AccountName"):SetData(tOld:FindChild("AccountName"):GetData())
	tNew:FindChild("AccountDisplayName"):SetText(tOld:FindChild("AccountDisplayName"):GetText())
	tNew:FindChild("AccountDisplayName"):SetTextColor(tOld:FindChild("ExtraData"):GetText())
	tNew:FindChild("ExtraData"):SetText(tOld:FindChild("ExtraData"):GetText())
	tNew:FindChild("ExtraData"):SetData(tOld:FindChild("ExtraData"):GetData())
end

--------------------------------------------------------------
--Parses through the message to get 1 line of text
--------------------------------------------------------------
function WhisperCatch:GetMessage(tMessage)
	local strNewMessage = ""
	
	--for all sections of text, concat onto message
	for idx, tChat in pairs(tMessage) do
		strNewMessage = strNewMessage .. tChat.strText
	end
	
	return strNewMessage 
end

--------------------------------------------------------------
--Creates a line of text in tTextBoxes for the player name
--------------------------------------------------------------
function WhisperCatch:GenerateMessage(strName, strRealm, strMessage, strType, strToFrom)
	--If there is a chat box open put the new line into it

	if self.tTextBoxes[strName] == nil then
		self.tTextBoxes[strName] = Apollo.LoadForm(self.xmlDoc, "Chat", self.wndMain, self)
		self.tTextBoxes[strName]:Show(false)
		self.tTextBoxes[strName]:SetData(strType)
	end
	local nCount = 1
	
	self.bShowTimestamp = self.ChatAddon.bShowTimestamp
	local tm = GameLib.GetLocalTime()
	local strTime = "" if self.bShowTimestamp then strTime = string.format("%d:%02d ", tm.nHour, tm.nMinute) end
	
	self.tChatLines[nCount] = Apollo.LoadForm("WhisperCatch.xml", "ChatLine", self.tTextBoxes[strName], self)
	self.tChatLines[nCount]:Show(false)
	self.tChatLines[nCount]:SetText(strMessage)
	self.tChatLines[nCount]:SetData(strToFrom)
	self.tChatLines[nCount]:FindChild("HeldData"):SetText(strTime)
	WhisperCatch:Resize(self.tChatLines, self.tTextBoxes)
	
	if self.tChatBoxes[strName] ~= nil then 
		self.tChatBoxes[strName]:FindChild("Chat"):DestroyChildren()
		self:CopyText(strName)
	end
end

--------------------------------------------------------------
--Coppies the text from tTextBoxes into the newly opened chat window
--------------------------------------------------------------
function WhisperCatch:CopyText(strName)
	local tLines = self.tTextBoxes[strName]:GetChildren()
	local strType = self.tTextBoxes[strName]:GetData()
	local strTime = self.tTextBoxes[strName]:FindChild("HeldData"):GetText()
	
	--For all lines in the text box
	for idx, value in pairs(tLines) do
		local nCount = 1
		--Get Text
		local strMessage = tLines[idx]:GetText()
		local strToFrom = tLines[idx]:GetData()

		--Create line
		self.tChatLines[nCount] = Apollo.LoadForm(self.xmlDoc, "ChatLine", self.tChatBoxes[strName]:FindChild("Chat"), self)
		
		self.strFontSize = self.ChatAddon.strFontOption

		if strType == "Whisper" then
				strMessage =  "<l Font=\""..self.strFontSize.."\" TextColor=\"Magenta\">".. strTime .. "[Whisper]".." "..strToFrom.." ".."</l>" .. "<l Font=\""..self.strFontSize.."\" TextColor=\"White\">".. strName .."</l>" .. "<l Font=\""..self.strFontSize.."\" TextColor=\"Magenta\">"..": ".. strMessage .."</l>"
		elseif strType == "AccountWhisper" then
				strMessage =  "<l Font=\""..self.strFontSize.."\" TextColor=\"xkcdVibrantPurple\">".. strTime .. "[Account Whisper]".." "..strToFrom.." ".."</l>" .. "<l Font=\""..self.strFontSize.."\" TextColor=\"White\">".. strName .."</l>" .. "<l Font=\""..self.strFontSize.."\" TextColor=\"xkcdVibrantPurple\">"..": ".. strMessage .."</l>"
		end	
		
		--Set text and resize
		self.tChatLines[nCount]:SetText(strMessage)
		self:Resize(self.tChatLines, self.tChatBoxes)
	end
end

--------------------------------------------------------------
--Resizes the lines and sorts the chat window when a new 
--text line is created
--------------------------------------------------------------
function WhisperCatch:Resize(tLines, tBox)
	--For everyline sent set hight to proper height
	for idx, value in pairs(tLines) do
		tLines[idx]:SetHeightToContentHeight()
	end
	
	--For every chat box sent, arrange children and scroll down
	for idx, value in pairs(tBox) do
		tBox[idx]:FindChild("Chat"):ArrangeChildrenVert()
		tBox[idx]:FindChild("Chat"):SetVScrollPos(tBox[idx]:FindChild("Chat"):GetVScrollRange())
	end	
end
--------------------------------------------------------------
--Find the chat log by using the find input
--------------------------------------------------------------
function WhisperCatch:GetInputWnd()
	local chatWnds = self.ChatAddon.tChatWindows
	local wndEdit = nil
	
	for idx, wndCurrent in pairs(chatWnds) do
		if wndCurrent:FindChild("Input"):GetData() then
			wndEdit = wndCurrent:FindChild("Input")
			break
		end
	end	
		
	if wndEdit == nil then
		for idx, wndCurrent in pairs(chatWnds) do
			wndEdit = wndCurrent:FindChild("Input")
			break
		end
	end
	
	return wndEdit:GetParent()
end

--------------------------------------------------------------
--Open settings window, currently not implemented
--------------------------------------------------------------
function WhisperCatch:OnSettings()

	if self.wndSetting:IsShown() then 
		self.wndSetting:Show(false)
	else
		self.wndSetting:FindChild("DisplaySetting"):SetCheck(self.DisplayAWSetting)
		self.wndSetting:Show(false)--Change to true
	end
end

--------------------------------------------------------------
--Fires the whisper event into the main chat when clicking a name
--------------------------------------------------------------
function WhisperCatch:OnButtonClick( wndHandler, wndControl, eMouseButton )
	local strName = wndControl:GetParent():FindChild("DisplayName"):GetData()
	local strAccName = wndControl:GetParent():FindChild("AccountName"):GetText()
	local strAccRealm = wndControl:GetParent():FindChild("AccountName"):GetData()
	local strChatWndName = strName
	
	if strAccRealm ~= "" then
		strChatWndName = strAccName
	end
	
-- Use this when the setting is set up 
-- if self.tChatBoxes[strChatWndName] ~= nil and self.tChatBoxes[strChatWndName]:IsVisible() or 
   -- self.tChatBoxes[strChatWndName] ~= nil and self.AlwaysSwitchToWindowSetting == true then 

    --If the window exists 						   --if the window is the currently selected window
	if self.tChatBoxes[strChatWndName] ~= nil and self.tChatBoxes[strChatWndName]:IsVisible() then
			local MainChat = self:GetInputWnd()
	
			wndChat = self.tChatBoxes[strChatWndName]
			wndChat:Detach()
			wndChat:Show(true)
			MainChat:AttachTab(wndChat , true)
			wndChat:FindChild("Input"):SetFocus()
	else
		if strName ~= nil or (strAccName ~= "" and strAccRealm ~= nil) then	
			if strAccName ~= "" then
				Event_FireGenericEvent("Event_EngageAccountWhisper", strAccName, strName, strAccRealm)
			else
				Event_FireGenericEvent("GenericEvent_ChatLogWhisper", strName)
			end
		end
	end
end

function WhisperCatch:SwitchVisibleName(bAcc)
	if bAcc then
		for idx, value in pairs(self.tNames) do
			if self.tNames[idx]:FindChild("AccountName"):GetText() ~= "" then
				self.tNames[idx]:FindChild("AccountDisplayName"):Show(true)
				self.tNames[idx]:FindChild("DisplayName"):Show(false)
			end
		end	
	else
		for idx, value in pairs(self.tNames) do
			self.tNames[idx]:FindChild("AccountDisplayName"):Show(false)
			self.tNames[idx]:FindChild("DisplayName"):Show(true)
		end	
	end
end
---------------------------------------------------------------------------------------------------
-- ChatWindow Functions
---------------------------------------------------------------------------------------------------
--------------------------------------------------------------
--Closes the specific chat window but does not remove the text
--------------------------------------------------------------
function WhisperCatch:OnCloseChatWindow( wndHandler, wndControl, eMouseButton )
	local strName = wndControl:GetParent():GetText()
	
	if strName ~= nil then
		--Remove all text for this name
		if self.tTextBoxes[strName] ~= nil then
			self.tTextBoxes[strName]:Destroy()
			self.tTextBoxes[strName] = nil
		end
		
		--If there is an open chat window close it		
		if self.tChatBoxes[strName] ~= nil then
			self.tChatBoxes[strName]:Detach()
			self.tChatBoxes[strName]:Destroy()
			self.tChatBoxes[strName] = nil
		end
	end
end
-----------------------------------------------------------------------------------------------
-- WhisperCatchForm Functions
-----------------------------------------------------------------------------------------------
function WhisperCatch:CheckDisplayAW( wndHandler, wndControl, eMouseButton )
	if wndControl:IsChecked() then
		self.DisplayAWSetting = true
		self:SwitchVisibleName(true)
	else
		self.DisplayAWSetting = false
		self:SwitchVisibleName(false)
	end
end

--------------------------------------------------------------
--Called when return is pressed while editing the chat box
--------------------------------------------------------------
function WhisperCatch:OnChat( wndHandler, wndControl, strText )
	local strChannelType  = wndControl:GetParent():GetText()
	local strRealm = wndControl:GetParent():GetData()

	if strChannelType == "Party" then
		ChatSystemLib.Command("/p " .. strText)
	elseif strChannelType == "Zone" then
		ChatSystemLib.Command("/zone " .. strText)
	elseif strChannelType == GuildName then
		ChatSystemLib.Command("/g ".. strText)
	elseif strChannelType == GuildOfficer then
		ChatSystemLib.Command("/go ".. strText)
	elseif strChannelType == "Say" then
		ChatSystemLib.Command("/say ".. strText)
	elseif strChannelType == "Yell" then
		ChatSystemLib.Command("/yell ".. strText)
	elseif strChannelType == "PvP" then
		ChatSystemLib.Command("/pvp ".. strText)
	elseif strChannelType == "Trade" then
		ChatSystemLib.Command("/trade ".. strText)
	elseif strChannelType == "Realm" then
		wndControl:SetText("Cannot speak in Realm")
	elseif strChannelType == "Instance" then
		ChatSystemLib.Command("/instance ".. strText)
	elseif strChannelType == GuildWarpary then
		ChatSystemLib.Command("/wp ".. strText)
	elseif strChannelType == GuildWarOfficer then
		ChatSystemLib.Command("/wpo ".. strText)
	elseif strChannelType == "Advice" then
		ChatSystemLib.Command("/a ".. strText)
	elseif strChannelType == "NPC Channel" then
		wndControl:SetText("Cannot speak in NPC Channel")
	elseif strChannelType == "Datachron" then
		wndControl:SetText("Cannot speak in Datachron")
	elseif strChannelType == "Loot" then
		wndControl:SetText("Cannot speak in Loot")	
	elseif strRealm  ~= "" then
		ChatSystemLib.Command("/aw  " .. strChannelType .. " " .. strText)
	else
		ChatSystemLib.Command("/w  " .. strChannelType .. " " .. strText)
	end
	
	wndControl:SetText("")
end

---------------------------------------------------------------------------------------------------
-- NameObject Functions
---------------------------------------------------------------------------------------------------
--------------------------------------------------------------
--Removes the selected name from the list
--------------------------------------------------------------
function WhisperCatch:RemoveName( wndHandler, wndControl, eMouseButton )
	local strName = ""
	local strAccName = ""
	
	strName = wndControl:GetParent():FindChild("DisplayName"):GetData()
	strAccName = wndControl:GetParent():FindChild("AccountName"):GetText()
	
	if strName ~= nil then
		--Resort the names 
		self:RearrangeRemove(strName)
		self.wndCont:ArrangeChildrenVert()
		self.wndCont:SetVScrollPos(0)
	end
	
	if strAccName ~= "" then
		strName = strAccName
	end
	
	if strName ~= nil then
		--Remove all text for this name
		if self.tTextBoxes[strName] ~= nil then
			self.tTextBoxes[strName]:Destroy()
			self.tTextBoxes[strName] = nil
		end
		
		--If there is an open chat window close it		
		if self.tChatBoxes[strName] ~= nil then
			self.tChatBoxes[strName]:Detach()
			self.tChatBoxes[strName]:Destroy()
			self.tChatBoxes[strName] = nil
		end
	end
end

--------------------------------------------------------------
--Opens the chat for the specific conversation
--------------------------------------------------------------
function WhisperCatch:ShowChat( wndHandler, wndControl, eMouseButton )
	local strName = ""
	local strRealm = wndControl:GetParent():FindChild("AccountName"):GetData()
	
	--If there is data in the account name use it to open the account name window
	if wndControl:GetParent():FindChild("AccountName"):GetText() ~= "" then
		strName = wndControl:GetParent():FindChild("AccountName"):GetText()
	else
		strName = wndControl:GetParent():FindChild("DisplayName"):GetData()
	end
	
	local MainChat = self:GetInputWnd()
	
	if strName ~= nil then
		if self.tChatBoxes[strName] ~= nil then
			self.tChatBoxes[strName]:Detach()
			self.tChatBoxes[strName]:Show(true)
			
			local nOpacity = self.ChatAddon.nBGOpacity
			self.tChatBoxes[strName]:FindChild("BGArt"):SetBGColor(CColor.new(1.0, 1.0, 1.0, nOpacity))
			self.tChatBoxes[strName]:FindChild("BGArt_SidePanel"):SetBGColor(CColor.new(1.0, 1.0, 1.0, nOpacity))
			
			local bBGFade = self.ChatAddon.bEnableBGFade
			local bNCFade = self.ChatAddon.bEnableNCFade

			self.tChatBoxes[strName]:SetStyle("AutoFadeNC", bNCFade)
			if self.bEnableNCFade then self.tChatBoxes[strName]:SetNCOpacity(1) end

			self.tChatBoxes[strName]:SetStyle("AutoFadeBG", bBGFade)
			if self.bEnableBGFade then self.tChatBoxes[strName]:SetBGOpacity(1) end

			MainChat:AttachTab(self.tChatBoxes[strName], true)
		elseif self.tTextBoxes[strName] ~= nil then
			--Create chat window and fill it with the text stored in tTextBoxes
			self.tChatBoxes[strName] = Apollo.LoadForm(self.xmlDoc, "ChatWindow", nil, self)
			self.tChatBoxes[strName]:Show(true)
			self:CopyText(strName)
			
			local nOpacity = self.ChatAddon.nBGOpacity
			self.tChatBoxes[strName]:FindChild("BGArt"):SetBGColor(CColor.new(1.0, 1.0, 1.0, nOpacity))
			self.tChatBoxes[strName]:FindChild("BGArt_SidePanel"):SetBGColor(CColor.new(1.0, 1.0, 1.0, nOpacity))
			
			local bBGFade = self.ChatAddon.bEnableBGFade
			local bNCFade = self.ChatAddon.bEnableNCFade

			self.tChatBoxes[strName]:SetStyle("AutoFadeNC", bNCFade)
			if self.bEnableNCFade then self.tChatBoxes[strName]:SetNCOpacity(1) end

			self.tChatBoxes[strName]:SetStyle("AutoFadeBG", bBGFade)
			if self.bEnableBGFade then self.tChatBoxes[strName]:SetBGOpacity(1) end
			
			--Attach tab to main chat
			MainChat:AttachTab(self.tChatBoxes[strName], true)
			
			--Temporarily used in OnChat
			self.tChatBoxes[strName]:SetText(strName)
			self.tChatBoxes[strName]:SetData(strRealm)
		end
	end
end
---------------------------------------------------------------------------------------------------
-- ShowList Functions
---------------------------------------------------------------------------------------------------
function WhisperCatch:OnShowList( wndHandler, wndControl, eMouseButton )
	if wndControl:IsChecked() then
		self.wndMain:Show(true)
		Apollo.StopTimer("CheckList")
	else
		self.wndMain:Show(false)
		Apollo.StopTimer("CheckList")
	end
end

-----------------------------------------------------------------------------------------------
-- WhisperCatch Instance
-----------------------------------------------------------------------------------------------
local WhisperCatchInst = WhisperCatch:new()
WhisperCatchInst:Init()
